
include "path.sol"
include "tools.sol"


boolean GRAPHICAL
boolean WINDOWS = false
boolean LASTMOD = false
boolean LASTMODTIME
boolean PRIORITY
boolean FREQUENCY
text DEFAULT_FREQUENCY
int mapType
text mapname
text WEBADDRESS
text logname          // path and name of the log file
text basePath
text siteURL = ""
text topname = ""
text localRoot

enum  OUT_XML, OUT_TEXT, OUT_HTML 


text version = "Simple Map 1.4  (c) 2006-2007 D.G. Sureau - www.scriptol.com"


array logfile = {}


void display(text t)
	if GRAPHICAL = true
		logfile.push(t + "\n")
	else	
		print t		
	/if	
return

`` read an ".ini" file, parse lines and return a dict of options

void readIni(text inipath)

  array ini
  inipath = Path.changeExt(inipath, "ini")
  if not file_exists(inipath) return
  ini.load(inipath)
  if(ini.size() = 0)
    display(inipath + " file not loaded")
    return
  /if

  for text option in ini
    option = option.trim()        ` remove trailing spaces and eol
    int i = option.find("=")      ` parse a line
    if i = nil  
		i = option.find(":")
		if i = nil	continue          ` no valid couple key=value
	/if	
    text k = option[--i].trim()   ` key
    text v = option[i+1..].trim() ` value
    if k
	= "sitemap":		mapname = v
    = "logfile":		logname = v
	= "frequency":	DEFAULT_FREQUENCY = v
	= "graphical":	GRAPHICAL = (v in { "yes", "1", "true", "ok" })
    /if
  /for

return



void usage()
	print
	print version
	print ~~
---------- Generates an XML, HTML or text site map.\n
Usage: \n
    smap [options] url local-path\n 
or  smap [options] remote-path\n	
    url:         in the form: www.mysite.com\n
    local-path:  in the form: c:\\mydir\\index.html\n
    remote-path: in the form: www.mysite.com\\index.html\n
Options:\n
    -v   verbose, display what happens.\n
    -d   debug, output on screen instead.\n
    -m   add a lastmod tag with file date only.\n
    -l   add a lastmod tag in long form, with date and time.\n
    -p   add a priority tag with default value 0.2.\n
    -f   add a change frequency tag with default value in smap.ini.\n
    -t   generate a text sitemap file.\n
    -h   generate a html sitamap file.\n	
Output:\n
    sitemap.xml or the name in smap.ini, into the root of the site.\n
~~
	
	exit(1)
	
return

void options(int num, array args)

	int fileindex
	text smapdir

	GRAPHICAL = false
	VERBOSE = false
	DEBUG = false
	LASTMOD = false
	LASTMODTIME = false
	PRIORITY = false
	FREQUENCY = false
	
	mapType = OUT_XML
	fileindex = 1
	

	while fileindex < num 
		text option = args[fileindex]
		text opt = option[0]
		
		if (opt <> '-') and (opt <> '/') break
		
		for opt in option[ 1 .. ]
			if opt.lower()
			= "g" : GRAPHICAL = true
			= "v" : VERBOSE = true 
			= "d" : DEBUG = true
			= "l" : LASTMODTIME = true	// last modif date and time 
			= "m" : LASTMOD = true	// last modif date only
			= "p" : PRIORITY = true
			= "f" : FREQUENCY = true
			= "t": 	mapType = OUT_TEXT		// output a text file (default xml)
						mapname = Path.changeExt(mapname, ".txt")
						FREQUENCY = false
						PRIORITY = false
						LASTMOD = false
						LASTMODTIME = false
			= "h": 	mapType = OUT_HTML		// output a text file (default xml)
						mapname = Path.changeExt(mapname, ".html")
						FREQUENCY = false
						PRIORITY = false
						LASTMOD = false
						LASTMODTIME = false
			else 
				print option, "bad option"
				usage()
			/if
		/for
		fileindex + 1
	/while
	
	readIni(args[0])

	`` two parameters remaining?  url + local path
	if (num - fileindex) = 2
		WEBADDRESS = args[fileindex]
		if not hasProtocol(WEBADDRESS)
			WEBADDRESS = "http://" + WEBADDRESS
		/if	
		fileindex + 1
		if not Path.hasDir(mapname)
				mapname = Path.merge(smapdir, mapname)
		/if	
	else
		`` remote url
		WEBADDRESS = ""
	/if	
	
	if DEBUG and VERBOSE
		print "Web address in arguments is", WEBADDRESS
	/if	
	
	smapdir, logname = Path.splitFile(args[0])
	logname = Path.merge(smapdir, "smap.log")
	mapname = "sitemap.xml"
	
	text fpath = args[fileindex].toText()
	if not Path.exists(fpath)
		display(fpath + " not found")
		exit()
	/if

	basePath, topname = Path.splitFile(fpath)
	basePath = noSlash(basePath)
	
	print "PATHS  base:$basePath top:$topname path:$fpath"
	
	if hasProtocol(basePath)
		siteURL = getURL(basePath)
		if siteURL.length() = basePath.length() ? topname = fpath
		if WEBADDRESS <> ""
			if WEBADDRESS <> siteURL
				display("Error, URLs must match:")
				display("'" +WEBADDRESS + "' and '" + siteURL + "' differ")
			/if
		/if	
	else
		if WEBADDRESS <> ""
			siteURL = WEBADDRESS
		else
			display("Error, web adress missing...")
			usage()
		/if	
	/if
	
	if not hasProtocol(siteURL)
		siteURL = "http://" + siteURL
	/if	
	

return
