
``	Simple Map - XML, text or HTML Site Map  Generator
``	A free open-source Scriptol program,  by (c) Denis G. Sureau
``	http://www.scriptol.com/seo/      ...the webmaster's source.
     

include "path.sol"
include "libphp.sol"
include "tools.sol"
include "options.sol"
include "url.sol"
include "page.sol"

array siteList = {}  	// list of pages in full URL with site address + path + filename
dict siteTime = {}		// dict of last mod date+time for each URL 
array parsedList = {}	// list of pages with just local path + filename
array skipped			// page skipped once must be ignored again

text pageText
array siteMap = { }


void output(text t)
	siteMap.push(t)
return



`` get the part of tag inside quotes or double-quotes, thus the url

text extractLink(int off)
	text c
	text link = ""
	while off < pageText.length()
		c = pageText[off]
		if c = '='  continue
		if c = ' '  continue
		if c = "\""  : off + 1; break; /if
		if c = '\'' : off + 1; break; /if
		break
	let off + 1
	while off < pageText.length()
		c = pageText[off]
		if c = '\'' break
		if c = "\""  break
		if c = '>'  break
		link + c             ` anything else is part of the url
	let off + 1
return link.trim()


`` Test if page already scanned

boolean inList(text page)
	if not hasProtocol(page)
		page = Path.merge(siteURL, page)
	/if
	//print page, "in=", page in siteList
	if page in parsedList return true
return false	


`` Add page to list, even if not valid, to not scan it again

void addList(text page)
	if not hasProtocol(page)
		page = Path.merge(siteURL, page)
	/if
	if page in parsedList return 
	parsedList.push(page)
return 


`` add local file with path to list of all scanned pages

boolean addPage(text fname)
	
	fname =fname.trim()
	
	if fname in parsedList 
		if DEBUG and VERBOSE
			print fname , "already parsed"
		/if	
		return false
	/if	

	//print "pushed", fname
	//parsedList.display()
    parsedList.push(fname)		
	if DEBUG = true print  "Scanning page", fname

return true	


`` and to the final list of all pages in the site to be referenced
`` fname is the URL

boolean addLink(text fname)
	
	fname = fname.trim()

	if not validExtension(fname) return false
	
	// add site url if required
	if not hasProtocol(fname)
		fname = Path.merge(siteURL, fname)
	/if	
	
	text url = setURL(fname) 	// convert to slashes
	if url in siteList return false
	
	PageWeb pw
	text localname
	boolean flag
	localname, flag = localPath(fname)
	
	if localname in skipped return false
	
	//print "Check if indexable", localname
	pw.load(localname)
	if not pw.indexable()
		print localname, "skipped, NOINDEX"
		skipped.push(localname)
		return false
	/if	 
	
	//print "LOCAL", localname, "REAL", realname, 'FROM', fname
	
	siteList.push(url)
	//if DEBUG = true 
	print "Added link", url
	
return true	


void addTime(text fname, real lmod)

	fname = fname.trim()
	
	if not hasProtocol(fname)
		fname = Path.merge(siteURL, fname)
	/if	
	
	if siteTime[fname] <> nil  return 		// already in list
	if not validExtension(fname) return 
	siteTime[fname] = date("", lmod)
	
	if DEBUG = true print "Added time", fname, date("", lmod)
	
return

`` Get the URL from the text, skipping other chars

array getLinks(constant alias text localBase, constant alias text fname)
	array x = ()
	text localLink
	
	x.load(basePath + fname)

	//print "GETLINK ($localBase) ($basePath) ($fname)"

	array pageLinks = ()		`` local links found on this page
  
	`` removing ending special codes
	scan x let x[] = x[].toText().lower().rtrim()
	pageText = x.join("")            ` merging lines into a text
	int offset = 0
	int srcoff = 0
	int shifting = 0
	
	while forever
		if offset <> nil let offset = pageText.find("href=", shifting)
		if srcoff <> nil 
			srcoff = pageText.find("frame src", shifting)
			if srcoff = nil
				srcoff = pageText.find("frame  src", shifting)
				if srcoff <> nil let srcoff + 6
			else
				srcoff + 6
			/if	
			
		/if	
		//print fname, offset, srcoff

		if offset = nil
			if srcoff = nil	break
			shifting = srcoff
		else
			if srcoff = nil
				shifting = offset
			else
				if offset < srcoff
					shifting = offset
				else
					shifting = srcoff
				/if
			/if
		/if	
		
		
		shifting + 4
		text link = extractLink(shifting)
		//if (shifting - 4) = srcoff print ">", fname, "---", link		
		shifting + link.length()
	
		if link = "" 				continue
		if link[0] = '#' 		continue		// anchors are omitted
		if link in siteList 		continue		// page already scanned, don't add it to list
		if link in pageLinks 	continue		// page not scanned, but link already added 
		
		if "../" in link
			display(" ../ such path is not valid, use absolute URL instead (in " + link+").")
			continue
		/if
		
   
		if hasProtocol(link)
			text host = getURL(link)
			//print "LINK WITH PROTOCOL", link, host, "=", (host = siteURL)
			if host = siteURL	`` this is the same current host, add the link
				pageLinks.push(link)
				//if addLink(link) = true print "link", link
				addLink(link)
			/if
			continue	`` same host or not, go to next line
		/if
    
		`` local link, without web address, simple html file
		
		//print "GETLINK ", link, getcwd(), localBase, endWith(getcwd(), localBase)
		
		if endWith(getcwd(), localBase)
			localLink = link			
		else
			localLink = Path.merge(localBase, link)
		/if
		
		
		if WINDOWS let localLink = setWindows(localLink)		
		
		if file_exists(localLink)
			localLink = Path.merge(localBase, localLink) 
			pageLinks.push(localLink)					
			addLink(localLink) 					// remote address will be added 
		else
			if DEBUG or VERBOSE
				print localLink, "not found in", getcwd()
			/if
		/if	
	
  /while

return pageLinks



`` parse a web page

int scanPage(text localBase, text fname)
	int counter = 0		// number of link in this page
	text fullpath			// local path of this page
	real lmod				// time of last modification of this page
	text ndir				// the dir in the local path in a link
	text cdir				// the current directory
	text link
	text linkBase			// sub-dir for the link to create

	boolean ABSOLUTE = false
	boolean MYSITE = true		// the URL is that of your site

	//print "SCAN PAGE ENTER base=", localBase, " file=", fname, "has=", hasProtocol(fname)

 	if not parsable(fname) return 0

	// full URL
	// we have to adjust the directory since we are not at root in some cases
	
	if hasProtocol(fname)
		fullpath, MYSITE = localPath(fname)			// Remove remote part of url if present
		fname = getLocal(fname)

		localBase, fname = Path.splitFile(fname)
		
		if MYSITE = false return	counter
		ABSOLUTE = true
		
		link = Path.merge(siteURL, localBase)
		link = Path.merge(link, fname)
		//print "CREATE LINK base=",  localBase, "fname=", fname, "fullpath=", fullpath, "link", link

		text localDir = Path.merge(localRoot, localBase)
		if WINDOWS let localDir = setWindows(localDir)				
		chdir(localDir)
		linkBase = localBase		// the link to create keeps the base
		localBase = ""				// but we are inside the subdir
		//print "changed dir to", localDir, "now base", localBase

	else
		//print "GETCWD", getcwd(), localRoot
		if getcwd() <> localRoot
			localBase = getRelativePath(getcwd())		// retrieve local path
			//print "LOCAL", localBase, "in", getcwd()
		/if	
		
		fullpath = fname			// we are inside a sub-directory
		if WINDOWS let fullpath = setWindows(fullpath)	// but perhaps with slashes in the relative path			
		
		link = createLink(siteURL, localBase, fname)
		//print "NP CREATE base", localBase, "fname", fname, "full", fullpath, "link", link
		linkBase = localBase
	/if
	
	cdir = getcwd()
	if endWith(cdir, localBase) let localBase = ""
	
	if DEBUG or VERBOSE
		print "Scanning for links", fullpath, localBase, fname
	/if	
	  
	if not file_exists(fullpath)
		//print "SCANPAGE", fullpath, "notfound"
		display("   " + fullpath + " not found")
		addList(fullpath)
		return counter
	/if
	
	lmod = filetime(fullpath)
	//print "EXIST in", getcwd(), fullpath, " addPage:", link
	if addPage(link) = false 
		if VERBOSE = true print link, "already scanned"
		//parsedList.display()
		return counter	// already scanned
	else
		addTime(fullpath, lmod)	// add to the directory
	/if	
	
	if VERBOSE = true
		display(fullpath+":")
	/if	
		
	text k, v
	counter + 1
	//print "Extracting by getLinks, base=", localBase, "file=", fname
	array dep = getLinks(linkBase, fname)			// get local links in this page
	if dep = nil return counter
	
	//dep.display()
	
	if VERBOSE = true
		print dep.size(), "links found"
	/if

		
	`` Now processing each link if it is a local page and call this function again
		
	for text page in dep
		text filename
		filename, MYSITE = localPath(page)
		if MYSITE = false continue
			
		if filename = "" continue
		//print filename," ", page
		if inList(page) = true continue
			
		if hasProtocol(page)
			cdir = getcwd()
			chdir(basePath)
			counter + scanPage("", page)			// scan this URL
			chdir(cdir)
			continue
		/if
		//print "HASDIR", filename, Path.hasDir(filename)	
		if Path.hasDir(filename) 
			ndir, filename = Path.splitFile(filename)
			if ABSOLUTE
				ndir = Path.merge(basePath, ndir)
				//print "absolute", ndir
			/if
			cdir = getcwd()
			//print "CHANGE", ndir, page
			chdir(ndir)
			//print "go scan", cdir, filename
			counter + scanPage(Path.merge(localBase, ndir), filename)			// scan this page
			changeDir(cdir)
			//print "RETURN", cdir
		else
			cdir = getcwd()
			if ABSOLUTE
				ndir = basePath
				page = Path.merge(basePath, page)	// add the local path
				chdir(ndir)
				//print "absolute", ndir
			/if
			counter + scanPage(localBase, page)			// scan each link in this page
			if ABSOLUTE let changeDir(cdir)				
		/if	
	/for

return counter



void buildTag(text tagname, text value)
	if value = "" return
	output("     <" + tagname + ">" + value + "</" + tagname + ">")
return



`` from the list of files
`` build now a xml file

void buildTheXmlFile()

	text name

	output("<?xml version=\"1.0\" encoding=\"UTF-8\"?>") 
	output("<urlset xmlns=\"http://www.google.com/schemas/sitemap/0.84\" ")
	output("xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" ")
	output("xsi:schemaLocation=\"http://www.google.com/schemas/sitemap/0.84 ")
	output("http://www.google.com/schemas/sitemap/0.84/sitemap.xsd\">")
	
	for name in siteList
	
		output("  <url>")
		
		buildTag("loc", textToUTF8(name))	`` create a tag, content UTF-8 compatible

		if PRIORITY = true
			buildTag("priority", text(0.2))
		/if
	
		if (LASTMOD = true) or (LASTMODTIME = true)
			text lmod = siteTime[name]
			if LASTMODTIME = true
				lmod = lmod[ .. 9]			// take only the date, 10 left chars
			/if	
			buildTag("lastmod", lmod)
		/if
		
		if FREQUENCY = true
			buildTag("changefreq", DEFAULT_FREQUENCY)
		/if	
		
		output("  </url>")
	/for	

	output("</urlset>")
	
return


int levelOffset





`` create a tag, content UTF-8 compatible

void buildLink(text link)

	text linkpath, name	

	output("<li>")
	link = textToUTF8(link)
	linkpath, name = Path.splitFile(link)
	output("<a href=" + link + ">" + name + "</a>")	
	output("</li>")
	
return

// extract dir

void openSubdir(text dirname)
	output("<br>")
	output("<h4>")
	output(dirname)
	output("</h4>")
	output("<ul>")
return

void closeSubdir()
	output("</ul>")
return



text removeBase(text linkpath, text base)
	int l = base.length()
	
	//print "enter remove to ", linkpath[0 -- l] , "and",  linkpath[l + 1 .. ]
	
	if l <  linkpath.length()
		if linkpath[0 -- l] = base return linkpath[l + 1 .. ]
	/if	

return linkpath

`` test if the local part of the URL holds a directory

boolean hasDir(text name)
	int l = name.length()
	if l < 2 return false
	if name.find("/", 1) <> nil return true
return false	

`` get dir

text getMainDir(text name)
	if name.length() = 0 return ""
	int i = name.find("/")
	if i < 1 return ""
return name[ -- i]
	


// parse the list of link at offset
// extract files
// and get position of first subdir
// base is the url plus the current path but filename

void processDirs(text base)

	text page
	text name, linkpath
	boolean empty = true
	text currdir = ""
	text thisdir
	
	`` get all the files, display them
	
	for int i in 0 -- siteList.size()	
		if siteList[i] = "" continue
		page = siteList[i]
		name = removeBase(page, base)
		
		if not Path.hasDir(name)
			buildLink(page)
			siteList[i] = ""
		else
			empty = false
		/if	
	/for	

	if empty = true return

	// now we have only paths with dirs inside
	// for each dir dir in the list we have to call this function
	// this identify this dir and all other dirs

	for int i in 0 -- siteList.size()	
		if siteList[i] = "" continue
		page = siteList[i]
		name = removeBase(page, base)
	
		thisdir = getMainDir(name)
		if thisdir <> currdir
			openSubdir(thisdir)
			processDirs(base + "/" + thisdir)
			closeSubdir()	
			currdir = thisdir
		/if	
	/for

return


`` scan list of links
`` build a text array of page in sub-dirs
`` make an entry for file in current directory

void processRoot()

	text urlpart	
	text name
	text link
	
	// process files

	output("<br>")
	output("<h1>")
	output(siteURL)
	output("</h1>")

	scan siteList
		link = siteList[]
		urlpart, name = splitURL(link)
		if Path.hasDir(name) <> true
			buildLink(link)
			siteList[] = ""
		/if	
	/scan	

	processDirs(siteURL)

return

`` from the list of files
`` build now a html page
`` process file at the level
`` then get first sub-dir, and loop

void buildTheHtmlTree()
	text name

	output("<html>")
	output("<head>")
	output("</head>")
	output("<body>")

	levelOffset = siteURL.length()
	processRoot()

	output("</body>")
	output("</html>")
	
return




`` Main Simple Map function

int SimpleMap(int num, array args)
	
	boolean changed = false

	options(num, args)
	
	print version
	
	int counter = 0

	if hasProtocol(topname)
		addLink(topname)
	else	
		addLink(Path.merge(siteURL, topname))
	/if	
	
	if VERBOSE
		print "Web adress:", siteURL
	/if	

	text currentPath = getcwd()
	if currentPath <> basePath
		if VERBOSE = true 
			print "Leaving", currentPath
			print "Going to", basePath
		/if	
		changeDir(basePath)	
		changed = true
	/if	


	localRoot = getcwd()
	
	for text t in localRoot
		if t = "\\" 
			WINDOWS = true
			break
		/if
	/for		

	basePath = ""   	// we are in the path
	counter  + scanPage("", topname)
	
	display("")
	siteList.sort()
	
	if VERBOSE = true	print "Creating", mapname

	if mapType
	= OUT_XML:		buildTheXmlFile()
	= OUT_TEXT:		siteList.store(mapname) 		// a text file
	= OUT_HTML:		buildTheHtmlTree()
	/if
	
	if mapType <> OUT_TEXT
		file x = fopen(mapname, "w")
		for text t in siteMap
			x.write(t + "\n")
		/for
		x.close()	
	/if

	logfile.store(logname)	
	display( text(siteList.size()) + " pages added to " + mapname)

	if changed = true
		if VERBOSE = true print "now returning to original path", currentPath
		changeDir(currentPath)
	/if	

return 0

